/*  
	Extension:	Wikipedia formatting context menu for Mozilla based browsers
	Author: 	Bananeweizen, http://www.bananeweizen.de
	Based on:	bbcode extension of Jed Brown, http://jedbrown.net/
				PastQuote extension by Ben Basson (Cusser), http://cusser.net/extensions/
*/

var wbPrefs=null;
var wikipedia_stringBundle;
var wikipedia_Document = null;
var scrollTop;
var scrollHeight;
var currentBox;

var wikipedia = {
	init: function() {
		wikipedia_setupDocument(document);
		try{
			var menu = document.getElementById('contentAreaContextMenu');
			menu.addEventListener('popupshowing', wikipedia.showHide, false);
		}catch (e){
			//Foobar!
		}
	},
	
	showHide: function() {
		var kind=0;
		var activate=true;
		var condition="wiki";
		try {
			wbPrefs=getPreferences();
			kind=wbPrefs.getIntPref("activation.kind");
			condition=wbPrefs.getCharPref("activation.condition");
		}
		catch (e) {
			alert(e);
		}
		if (kind==1) {
			activate=(window.content.location.toString().indexOf(condition)>=0);
		}
		document.getElementById('context-wikipedia').hidden = document.getElementById('context-undo').hidden || !activate;
	},
	
	getClipboard: function() {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip){
			return "";
		}
		
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans){
			return "";
		}

		trans.addDataFlavor("text/unicode");
		clip.getData(trans,clip.kGlobalClipboard);

		var str_clipboard=new Object();
		var strLength=new Object();
		try{
			trans.getTransferData("text/unicode",str_clipboard,strLength);
			if (str_clipboard){
				str_clipboard=str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
			}
		}catch (e){
			//alert("No text in the clipboard, please copy something first.");
		}
		return str_clipboard;
	},
	
	getSelection: function() {
		currentBox = document.commandDispatcher.focusedElement;
		scrollTop = currentBox.scrollTop;
		scrollHeight = currentBox.scrollHeight;

		//Get Selected text (if selected)
		var selStartPos = currentBox.selectionStart;
		var selEndPos = currentBox.selectionEnd;
		str = currentBox.value.substring(selStartPos, selEndPos);
		
		return str;
	},

	getSelectionAll: function() {
		currentBox.setSelectionRange(0,currentBox.value.length-1);
		return wikipedia.getSelection();
	},
	
	getForcedSelection: function(format) {
		var str = wikipedia.getSelection();
		if (str==null || str=="") {
			str=getInput("wikipedia.input."+format);
		}
		return str;
	},

	insert: function(someText) {
		wikipedia.getSelection();
		wikipedia.insertAtCursor(someText);
	},
	
	surround: function(format, left,right) {
		var str=wikipedia.getForcedSelection(format);
		var leftTrim="";
		while ((str.length>0) && (str.substring(0,1) == ' ')) {
			leftTrim=leftTrim+" ";
			str=str.substring(1,str.length);
		}
		var rightTrim="";
		while ((str.length>0) && (str.substring(str.length-1,str.length) == ' ') ) {
			rightTrim=rightTrim+" ";
			str=str.substring(0,str.length-1);
		}
		wikipedia.insertAtCursor(leftTrim+left+str+right+rightTrim);
	},
	
	paragraph: function(format,markup) {
		var str=wikipedia.getForcedSelection(format);
		wikipedia.insertAtCursor("\n"+markup+" "+trim(str)+" "+markup+"\n");
	},
	
	changeParagraphs: function(change) {
		var str=wikipedia.getSelection();
		if (str=="") {
			str=wikipedia.getSelectionAll();
		}
		var lines=str.split("\n");
		var insert="";
		for (i=0;i<lines.length;i=i+1) {
			var line=lines[i];
			if (line.indexOf("==")>=0) {
				if (change==1) {
					line = line.replace(new RegExp ('====', 'gi'), '=====') ;
					line = line.replace(new RegExp ('===', 'gi'), '====') ;
					line = line.replace(new RegExp ('==', 'gi'), '===') ;
				}
				else {
					line = line.replace(new RegExp ('===', 'gi'), '==') ;
					line = line.replace(new RegExp ('====', 'gi'), '===') ;
					line = line.replace(new RegExp ('=====', 'gi'), '====') ;
				}
			}
			if (i>0) {
				insert=insert+"\n";
			}
			insert=insert+line;
		}
		wikipedia.insertAtCursor(insert);
	},
	
	insertSignature: function() {
		var trailing="";
		wikipedia.getSelection();
		var selStartPos = currentBox.selectionStart;
		if (selStartPos>0) {
			var before = currentBox.value.substring(selStartPos-1, selStartPos);
			if ((before!=' ') && (before!='\n')) {
				trailing=" ";
			}
		}
		wikipedia.insert(trailing+'--~~~~');
	},
  
	insertAtCursor: function(aText) {
		try {
			var command = "cmd_insertText";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				params.setStringValue("state_data", aText);
				controller.doCommandWithParams(command, params);
				
				//restore position
				var nHeight = currentBox.scrollHeight - scrollHeight;
				currentBox.scrollTop = scrollTop + nHeight;
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
  
	insertList: function(listSign) {
		var str=wikipedia.getSelection();
		var lines=str.split("\n");
		var insert="";
		for (i=0;i<lines.length;i=i+1) {
			if (i>0) {
				insert=insert+"\n";
			}
			if (trim(lines[i])!="") {
				insert=insert+listSign+lines[i];
			}
		}
		for (i=lines.length;i<3;i=i+1) {
			insert=insert+"\n"+listSign;
		}
	  	wikipedia.insertAtCursor(insert);
	},
	
	insertDefinitionList: function() {
		var topic=wikipedia_stringBundle.getString("wikipedia.definitionlist.topic");
		var definition=wikipedia_stringBundle.getString("wikipedia.definitionlist.definition");
		var def=";"+topic+":"+definition+"\n";
		wikipedia.insertAtCursor(def+def);
	},
	
	removeList: function() {
		var lines=wikipedia.getSelection().split("\n");
		for (i=0;i<lines.length;i=i+1) {
			if (lines[i].length>0) {
				var c=lines[i].substring(0,1);
				if ((c=='*') || (c=='#')) {
					lines[i]=lines[i].substr(1,lines[i].length-1);
				}
			}
		}
		var insert=lines[0];
		for (i=1;i<lines.length;i=i+1) {
			insert=insert+"\n"+lines[i];
		}
	  	wikipedia.insertAtCursor(insert);
	},
	
	insertTable: function() {
		var str=wikipedia.getSelection();
        var ok = [false];
        var params = [0,0,trim(str),false,false,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/table.xul", "WikiTable","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var rows=params[0];
        	var columns=params[1];
        	var content=params[2];
        	var border=params[3];
        	var headline=params[4];
        	var table="{|";
        	if (border) {
        		table=table+" border=\"1\"";
        	}
        	table=table+"\n";
        	for (r=0;r<rows;r=r+1) {
        		var cellStart="|";
        		if (headline && (r==0)) {
        			cellStart="!";
        		}
        		for (c=0;c<columns;c=c+1) {
        			table=table+cellStart+content+"\n";
        		}
        		if (r<rows-1) {
	        		table=table+"|-\n";
	        	}
        	}
        	table=table+"|}\n";
			wikipedia.insertAtCursor(table);
        }
	},
	
	insertImage: function() {
		var str=wikipedia.getSelection();
        var ok = [false];
        // use selected string as either image name or description
        var imageName="";
        var imageDesc="";
        if (str.indexOf(".")>=0) {
        	imageName=trim(str);
        }
        else {
        	imageDesc=trim(str);
        }
        // call dialog
        var params = [imageName,imageDesc,true,false,false,false,0,false,false,false,0,0,0,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/image.xul", "WikiImage","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var file=params[0];
        	var description=params[1];
        	var thumb=params[2];
        	var thumbsized=params[3];
        	var framed=params[4];
        	var full=params[5];
        	var pixels=params[6];
        	var left=params[7];
        	var center=params[8];
        	var right=params[9];
        	var image="[["+wikipedia_stringBundle.getString("wikipedia.image.namespace")+":"+file;
        	if (framed) {
        		image=image+"|framed";
        	}
        	else if (thumb) {
        		image=image+"|thumb";
        	}
        	else if (thumbsized) {
        		image=image+"|thumb|"+pixels+"px";
        	}
        	if (left) {
        		image=image+"|left";
        	}
        	else if (right) {
        		image=image+"|right";
        	}
        	else if (center) {
        		image=image+"|none";
        	}
        	image=image+"|"+description;
        	image=image+"]]";
        	if (center) {
        		image="<center>"+image+"</center>";
        	}
			wikipedia.insertAtCursor(image);
        }
	},
	
	insertLink: function() {
		var str=wikipedia.getSelection();
        var ok = [false];
        var params = [trim(str),"",0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/link.xul", "WikiLink","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var target=params[0];
        	var label=params[1];
        	var link="[["+target;
        	if (label!="") {
        		link=link+"|"+label;
        	}
        	link=link+"]]";
			wikipedia.insertAtCursor(link);
        }
	}
}

function trim(s) {
	if (s==null || s=="") {
		return "";
	}
	while (s.substring(0,1) == ' ') {
		s = s.substring(1,s.length);
	}
	while (s.substring(s.length-1,s.length) == ' ') {
		s = s.substring(0,s.length-1);
	}
	return s;
}

function getInput(textId) {
	var dialogText=wikipedia_stringBundle.getString(textId);
	var newtext = prompt(dialogText);
	return newtext;
}

function getPreferences() {
	if (!wbPrefs) {
		try {
			var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			wbPrefs = prefService.getBranch("wikipedia."); // preferences wikipedia node
		} catch (e) {
			alert(e);
		}
	}
	return wbPrefs;
}
//May not be needed, but just incase.
function wikipedia_setupDocument(aDoc)
{
	wikipedia_Document = aDoc;
	wikipedia_stringBundle = aDoc.getElementById("wikipediastrings");
}

//Make sure wikipedia loads on start-up of browser.
window.addEventListener('load', wikipedia.init, false);